@@ -2,7 +2,7 @@
  * Oroborus Window Manager
  *
  * Copyright (C) 2001 Ken Lynch
- * Copyright (C) 2002 Stefan Pfetzing
+ * Copyright (C) 2002-2005 Stefan Pfetzing
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,38 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
  */
 
-#include "oroborus.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <X11/Xlib.h>
+#include <X11/extensions/shape.h>
+#include <X11/extensions/xf86vmode.h>
+#include <X11/keysym.h>
+#include <X11/Xatom.h>
+#include <X11/xpm.h>
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
+#include <signal.h>
+
+#include "client.h"
+#include "events.h"
+#include "frame.h"
+#include "getopt.h"
+#include "hints.h"
+#include "i18n.h"
+#include "keyboard.h"
+#include "misc.h"
+#include "pixmap.h"
+#include "workspaces.h"
+
+#include "globals.h"
 
 void
-clientInitPosition (Client * c)
+clientInitPosition (client_t * c)
 {
   int mx, my, min_x, min_y, max_x, max_y;
 
@@ -70,7 +98,7 @@ clientInitPosition (Client * c)
 }
 
 void
-clientGrabKeys (Client * c)
+clientGrabKeys (client_t * c)
 {
   int i;
 #ifdef DEBUG
@@ -78,10 +106,10 @@ clientGrabKeys (Client * c)
   printf ("grabbing keys for client (%#lx)\n", c->window);
 #endif
 
-  // do not grab the keys for panels.
+  /* do not grab the keys for panels. */
   if (c->non_focusing)
     {
-      // bind only these keys to "special" windows
+      /* bind only these keys to "special" windows */
       int keynums[] = {
 	KEY_CYCLE_WINDOWS,
 	KEY_NEXT_WORKSPACE,
@@ -104,12 +132,12 @@ clientGrabKeys (Client * c)
       return;
     }
 
-  for (i = 0; i < KEY_COUNT; i++)
+  for (i = 0; i < KEY_ENUM_MAX; i++)
     grabKey (dpy, &keys[i], c->window);
 }
 
 void
-clientUngrabKeys (Client * c)
+clientUngrabKeys (client_t * c)
 {
 #ifdef DEBUG
   printf ("entering clientUngrabKeys\n");
@@ -120,7 +148,7 @@ clientUngrabKeys (Client * c)
 }
 
 void
-clientGravitate (Client * c, int mode)
+clientGravitate (client_t * c, int mode)
 {
   int gravity, dx = 0, dy = 0;
 
@@ -174,7 +202,7 @@ clientGravitate (Client * c, int mode)
 }
 
 void
-clientAddToList (Client * c)
+clientAddToList (client_t * c)
 {
   int i;
   CARD32 *new;
@@ -183,7 +211,7 @@ clientAddToList (Client * c)
   printf ("entering clientAddToList\n");
 #endif
 
-  new = (CARD32 *) xmalloc (sizeof (CARD32) * client_count);
+  new = (CARD32 *) malloc (sizeof (CARD32) * client_count);
   if (new)
     {
       for (i = 0; i < client_count - 1; i++)
@@ -203,7 +231,7 @@ clientAddToList (Client * c)
 }
 
 void
-clientRemoveFromList (Client * c)
+clientRemoveFromList (client_t * c)
 {
   int i, j;
   CARD32 *new;
@@ -212,7 +240,7 @@ clientRemoveFromList (Client * c)
   printf ("entering clientRemoveFromList\n");
 #endif
 
-  new = (CARD32 *) xmalloc (sizeof (CARD32) * client_count);
+  new = (CARD32 *) malloc (sizeof (CARD32) * client_count);
   if (new)
     {
       for (i = 0, j = 0; i < client_count; i++)
@@ -234,7 +262,7 @@ clientRemoveFromList (Client * c)
 }
 
 int
-clientGetWidthInc (Client * c)
+clientGetWidthInc (client_t * c)
 {
   if (c->size->flags & PResizeInc)
     return c->size->width_inc;
@@ -242,7 +270,7 @@ clientGetWidthInc (Client * c)
 }
 
 int
-clientGetHeightInc (Client * c)
+clientGetHeightInc (client_t * c)
 {
   if (c->size->flags & PResizeInc)
     return c->size->height_inc;
@@ -250,7 +278,7 @@ clientGetHeightInc (Client * c)
 }
 
 void
-clientSetWidth (Client * c, int w1)
+clientSetWidth (client_t * c, int w1)
 {
   int w2;
 
@@ -273,7 +301,7 @@ clientSetWidth (Client * c, int w1)
 }
 
 void
-clientSetHeight (Client * c, int h1)
+clientSetHeight (client_t * c, int h1)
 {
   int h2;
 
@@ -295,19 +323,19 @@ clientSetHeight (Client * c, int h1)
   c->height = h1;
 }
 
-Client *
+client_t *
 clientGetTopMost (int layer)
 {
   return clientGetTopMostShaded (layer, 0);
 }
 
-Client *
+client_t *
 clientGetTopMostShaded (int layer, int non_shaded)
 {
   Window w1, w2, *wins;
   unsigned int i, count;
   XWindowAttributes attr;
-  Client *top = NULL, *c;
+  client_t *top = NULL, *c;
 
 #ifdef DEBUG
   printf ("entering clientGetTopMost\n");
@@ -336,13 +364,13 @@ clientGetTopMostShaded (int layer, int non_shaded)
   return top;
 }
 
-Client *
+client_t *
 clientGetBottomMost (int layer)
 {
   Window w1, w2, *wins;
   unsigned int i, count;
   XWindowAttributes attr;
-  Client *c = NULL;
+  client_t *c = NULL;
 
 #ifdef DEBUG
   printf ("entering clientGetBottomMost\n");
@@ -366,10 +394,10 @@ clientGetBottomMost (int layer)
 }
 
 void
-clientConfigure (Client * c, XWindowChanges * wc, int mask)
+clientConfigure (client_t * c, XWindowChanges * wc, int mask)
 {
   XConfigureEvent ce;
-  Client *sibling = NULL;
+  client_t *sibling = NULL;
 
 #ifdef DEBUG
   printf ("entering clientConfigure\n");
@@ -450,7 +478,7 @@ clientConfigure (Client * c, XWindowChanges * wc, int mask)
 void
 clientFrame (Window w)
 {
-  Client *c;
+  client_t *c;
   XWindowAttributes attr;
   XWindowChanges wc;
   long dummy;
@@ -463,7 +491,7 @@ clientFrame (Window w)
   printf ("framing client (%#lx)\n", w);
 #endif
 
-  c = xmalloc (sizeof (*c));
+  c = malloc (sizeof (*c));
 
   c->size = XAllocSizeHints ();
   XGetWMNormalHints (dpy, w, c->size, &dummy);
@@ -484,7 +512,7 @@ clientFrame (Window w)
   c->width = c->old_width = attr.width;
   c->height = c->old_height = attr.height;
   c->border_width = attr.border_width;
-  for (i = 0; i < BUTTON_COUNT; i++)
+  for (i = 0; i < BUTTON_ENUM_MAX; i++)
     c->button_pressed[i] = False;
   getNetWMStrut (w, margins, c->margins);
   if (!getGnomeHint (w, win_hints, &c->win_hints))
@@ -581,7 +609,7 @@ clientFrame (Window w)
   for (i = 0; i < 4; i++)
     XDefineCursor (dpy, c->corners[i], resize_cursor[i]);
 
-  for (i = 0; i < BUTTON_COUNT; i++)
+  for (i = 0; i < BUTTON_ENUM_MAX; i++)
     c->buttons[i] = XCreateSimpleWindow (dpy, c->frame, 0, 0, 1, 1, 0, 0, 0);
 
   if (clients)
@@ -632,7 +660,7 @@ clientFrame (Window w)
 }
 
 void
-clientUnframe (Client * c, int remap)
+clientUnframe (client_t * c, int remap)
 {
 #ifdef DEBUG
   printf ("entering clientUnframe\n");
@@ -698,7 +726,7 @@ clientFrameAll ()
 void
 clientUnframeAll ()
 {
-  Client *c;
+  client_t *c;
   unsigned int count, i;
   Window w1, w2, *wins = NULL;
 
@@ -717,10 +745,10 @@ clientUnframeAll ()
     XFree (wins);
 }
 
-Client *
+client_t *
 clientGetFromWindow (Window w, int mode)
 {
-  Client *c;
+  client_t *c;
   int i;
 
 #ifdef DEBUG
@@ -743,7 +771,7 @@ clientGetFromWindow (Window w, int mode)
 }
 
 void
-clientShow (Client * c, int change_state)
+clientShow (client_t * c, int change_state)
 {
 #ifdef DEBUG
   printf ("entering clientShow\n");
@@ -760,7 +788,7 @@ clientShow (Client * c, int change_state)
 }
 
 void
-clientHide (Client * c, int change_state)
+clientHide (client_t * c, int change_state)
 {
 #ifdef DEBUG
   printf ("entering clientHide\n");
@@ -775,7 +803,7 @@ clientHide (Client * c, int change_state)
 }
 
 void
-clientClose (Client * c)
+clientClose (client_t * c)
 {
 #ifdef DEBUG
   printf ("entering clientClose\n");
@@ -786,20 +814,25 @@ clientClose (Client * c)
 }
 
 void
-clientKill (Client * c)
+clientKill (client_t * c)
 {
+  int pid = getNetWMPid (c->window);
+
 #ifdef DEBUG
   printf ("entering clientKill\n");
 #endif
+  
+  if (pid)
+    kill (pid, SIGKILL);
 
   XKillClient (dpy, c->window);
 }
 
 void
-clientRaise (Client * c)
+clientRaise (client_t * c)
 {
   XWindowChanges wc;
-  Client *c2;
+  client_t *c2;
   int i;
 
 #ifdef DEBUG
@@ -830,7 +863,7 @@ clientRaise (Client * c)
 }
 
 void
-clientLower (Client * c)
+clientLower (client_t * c)
 {
   XWindowChanges wc;
 
@@ -844,7 +877,7 @@ clientLower (Client * c)
 }
 
 void
-clientSetLayer (Client * c, int l)
+clientSetLayer (client_t * c, int l)
 {
   int old_layer = c->win_layer;
 
@@ -865,7 +898,7 @@ clientSetLayer (Client * c, int l)
 }
 
 void
-clientSetWorkspace (Client * c, int ws)
+clientSetWorkspace (client_t * c, int ws)
 {
 #ifdef DEBUG
   printf ("entering clientSetWorkspace\n");
@@ -893,7 +926,7 @@ clientSetWorkspace (Client * c, int ws)
 }
 
 void
-clientSetState (Client * c, CARD32 mask, CARD32 value)
+clientSetState (client_t * c, CARD32 mask, CARD32 value)
 {
   XWindowChanges wc;
 
@@ -913,7 +946,7 @@ clientSetState (Client * c, CARD32 mask, CARD32 value)
 }
 
 void
-clientToggleShaded (Client * c)
+clientToggleShaded (client_t * c)
 {
   XWindowChanges wc;
 
@@ -930,7 +963,7 @@ clientToggleShaded (Client * c)
 }
 
 void
-clientToggleAbove (Client * c)
+clientToggleAbove (client_t * c)
 {
 #ifdef DEBUG
   printf ("entering clientToggleAbove\n");
@@ -953,7 +986,7 @@ clientToggleAbove (Client * c)
 }
 
 void
-clientToggleSticky (Client * c)
+clientToggleSticky (client_t * c)
 {
 #ifdef DEBUG
   printf ("entering clientToggleSticky\n");
@@ -966,7 +999,7 @@ clientToggleSticky (Client * c)
 }
 
 void
-clientToggleFullscreen (Client * c)
+clientToggleFullscreen (client_t * c)
 {
   XWindowChanges wc;
 
@@ -975,7 +1008,7 @@ clientToggleFullscreen (Client * c)
   printf ("fullscreening/unfullscreening client (%#lx)\n", c->window);
 #endif
 
-  // check if the window is already at its maximum size.
+  /* check if the window is already at its maximum size. */
   if (c->size->flags & (PMinSize | PMaxSize) &&
       c->size->min_width == c->size->max_width &&
       c->size->min_height == c->size->max_height)
@@ -1006,10 +1039,14 @@ clientToggleFullscreen (Client * c)
     }
 
   clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight);
+  
+  /* make sure the client is focussed and raised afterwards. */
+  clientSetFocus (c, True);
+  clientRaise (c);
 }
 
 void
-clientToggleMaximized (Client * c, int mode)
+clientToggleMaximized (client_t * c, int mode)
 {
   XWindowChanges wc;
 
@@ -1079,9 +1116,9 @@ clientToggleMaximized (Client * c, int mode)
 }
 
 void
-clientSetFocus (Client * c, int sort)
+clientSetFocus (client_t * c, int sort)
 {
-  Client *tmp;
+  client_t *tmp;
   int i;
 
 #ifdef DEBUG
@@ -1148,11 +1185,10 @@ clientSetFocus (Client * c, int sort)
     }
 }
 
-Client *
+client_t *
 clientGetFocus ()
 {
-  Client *c;
-//  Window w;
+  client_t *c;
   int i;
 
 #ifdef DEBUG
@@ -1162,13 +1198,11 @@ clientGetFocus ()
   for (c = clients, i = 0; i < client_count; c = c->next, i++)
     if (c->focus && c->win_workspace == workspace)
       return c;
-//  XGetInputFocus (dpy, &w, &dummy);
-//  return clientGetFromWindow (w, WINDOW);
   return NULL;
 }
 
 void
-clientDrawOutline (Client * c)
+clientDrawOutline (client_t * c)
 {
 #ifdef DEBUG
   printf ("entering clientDrawOutline\n");
@@ -1179,10 +1213,10 @@ clientDrawOutline (Client * c)
 }
 
 void
-clientMove (Client * c, XEvent * e)
+clientMove (client_t * c, XEvent * e)
 {
   int mx, my, moving = True, grab = False, use_keys = False, g1, g2;
-  Client *closestx, *closesty, *c2;
+  client_t *closestx, *closesty, *c2;
   XWindowChanges wc;
   XEvent ev;
   int i;
@@ -1469,7 +1503,7 @@ clientMove (Client * c, XEvent * e)
 }
 
 void
-clientResize (Client * c, XEvent * e)
+clientResize (client_t * c, XEvent * e)
 {
   int mx, my, oldw, oldh, resizing = True, grab = False, corner =
     CORNER_BOTTOM_RIGHT, use_keys = False, g1, g2;
@@ -1659,10 +1693,10 @@ clientResize (Client * c, XEvent * e)
   XUngrabServer (dpy);
 }
 
-Client *
-clientGetNext (Client * c, int mask)
+client_t *
+clientGetNext (client_t * c, int mask)
 {
-  Client *c2;
+  client_t *c2;
   unsigned int i, okay;
 
 #ifdef DEBUG
@@ -1712,9 +1746,9 @@ clientGetNext (Client * c, int mask)
 }
 
 void
-clientCycle (Client * c)
+clientCycle (client_t * c)
 {
-  Client *c2 = c;
+  client_t *c2 = c;
   int cycling = True, g1, g2;
   XEvent ev;
 
@@ -1806,7 +1840,7 @@ clientCycle (Client * c)
 }
 
 void
-clientButtonPress (Client * c, Window w, XButtonEvent * bev)
+clientButtonPress (client_t * c, Window w, XButtonEvent * bev)
 {
   int pressed = True, b, g1, g2;
   XEvent ev;
@@ -1815,7 +1849,7 @@ clientButtonPress (Client * c, Window w, XButtonEvent * bev)
   printf ("entering clientButtonPress\n");
 #endif
 
-  for (b = 0; b < BUTTON_COUNT; b++)
+  for (b = 0; b < BUTTON_ENUM_MAX; b++)
     if (c->buttons[b] == w)
       break;
 
@@ -1912,7 +1946,7 @@ clientButtonPress (Client * c, Window w, XButtonEvent * bev)
 }
 
 void
-clientWarpMouse (Client * c)
+clientWarpMouse (client_t * c)
 {
   if (!c)
     return;
@@ -1924,8 +1958,8 @@ clientWarpMouse (Client * c)
 		  (frameWidth (c) / 2), (frameHeight (c) / 2));
 }
 
-int
-clientInBorderlessList (Client * c)
+Bool
+clientInBorderlessList (client_t * c)
 {
   XClassHint *class_res;
   char *win_class_name;
@@ -1953,7 +1987,7 @@ clientInBorderlessList (Client * c)
     }
 
   strSize = sizeof (char) * (strlen (borderless_windows) + 1);
-  buf = xmalloc (strSize);
+  buf = malloc (strSize);
   snprintf (buf, strSize, "%s", borderless_windows);
 
   tok = strtok (buf, ";");
